function [varargout] = main_HANK_ss4(pi,firstcall)

%------------------------------------------------------------------------
%
% main_HANK_ss4
%
% October 2021
%
% Replication codes: "Monetary Policy and Capital Misallocation"
%
% (c) Beatriz Gonzalez, Galo Nuno, Dominik Thaler and Silvia Albrizio
% 
% This file solves for the SS in HANK for a given pi (input)
%
%     - firstcall = 0 if we are solving for initial SS where we normalize 
%       upsilon st L=1
%
%     - firstcall = 1 if upsilon is treated as a parameter,L is endogenous
%
% Includes function SS_given_L, residual and findzstar
%
%------------------------------------------------------------------------

%initiate params
global params

pi
rho       = params(1);
rho_hh    = params(2);
delta     = params(3);
gamma     = params(4);
alpha     = params(5);
psi       = params(6);
sig       = params(7);
theta     = params(8);
Deltat    = params(9);
dz        = params(10);
Upsilon   = params(11);
thetaP    = params(12);
epsilon   = params(13);
phiTR     = params(14); 
pibar     = params(15);
eta_q     = params(16);
timeless  = params(17);
share_d   = params(18);
rhoTR     = params(19);
phiK      = params(20);
J         = params(21);
IP        = params(23);

dz2       = dz^2;

z         = params(end-4*J+1:end-3*J);
varrho    = params(end-3*J+1:end-2*J);
xi        = params(end-2*J+1:end-1*J);
beta_base = params(end-1*J+1:end);

sig2  = sig^2;       %  variance shock process in logs

% -- PRELIMINARIES  --------------------
maxit = 10000;
tol   = 1e-10;

opt     = optimoptions('fsolve','FunctionTolerance',1e-13,'OptimalityTolerance',1e-13,'Display','off');

% Preallocation
c=zeros(J,1);s=zeros(J,1);shat=zeros(J,1);
omega=zeros(J,1);
f0=zeros(J,1);cdf0=zeros(J,1);f_d=zeros(J,1);
cdf=zeros(J,1) ;onecdf=zeros(J,1);
B=zeros(J,J);
beta_add=zeros(J,1);
y=zeros(J,1); Xaux_vec=zeros(J,1); X=0;Xaux=0;xstar=0;varphi=0;Zaux=0;
A=0;K=0;D=0;w=0;C=0;Y=0;T=0;

% Initialize guesses
L       = 1;                       % Normalization of labor supply Ls=1  
f0(:)   = lognpdf(z,0,sqrt(sig2)); % Guess for initial distribution
f0      = f0 ./sum(dz*f0);
fnew    = f0;

% % Boundary conditions
beta_base(1) = beta_base(1)+xi(1);   
beta_base(J) = beta_base(J)+varrho(J);

% Construct part of B Matrix that is invariant
B_base = spdiags(beta_base(:),0,J,J)+spdiags(varrho(:),-1,J,J)+spdiags([0;xi(2:J)],1,J,J);


% -- STEADY STATE  --------------------

r       = rho_hh;
i       = rho_hh + pi;
m       = (epsilon-1)/epsilon + rho_hh*pi*thetaP/epsilon;
mbar    = m/((epsilon-1)/epsilon); %=m if no subsidy, =m/((epsilon-1)/epsilon) if subsidy
R       = (rho_hh+delta);

zstar=.7;

SS_given_L;


if firstcall==0
    sol=fsolve(@residual,1,opt);   
    residual(sol);
end

f = f_d*dz; %This distribution is in [0,1]

% CHECKS
% This should be 1
sumationerror=sum(f_d*dz);
% This should be 0
sum(shat.*f_d*dz);


Z      = Zaux;
Omega  = -xstar/gamma+1;
phi    = varphi;
X      = Xaux;
i      = r+pi;


varargout{1}    = Omega;
varargout{2}    = phi;
varargout{3}    = w;
varargout{4}    = r;
varargout{5}    = A;
varargout{6}    = X;
varargout{7}    = K;
varargout{8}    = D;
varargout{9}    = Z;
varargout{10}   = C;
varargout{11}   = zstar;
varargout{12}   = L;
varargout{13}   = m;
varargout{14}   = i;
varargout{15}   = Y;
varargout{16}   = T;
varargout{17}   = sumationerror;
varargout{18}   = mbar;

for i=1:J
    varargout{end+1} = f_d(i);
end
varargout{end+1} = w*C^(-1); % Upsilon:parameter multiplying disutility of labor

if firstcall==1
    varargout{1} = cell2mat(varargout);
end

%----------------------------------------------------------------
function [out] =SS_given_L % to find distribution given L; 

    for n=1:maxit

        f_d  = fnew;

        % 1- Compute cdf
        if n==1;zstar=.7;end
        findzstar2   = @(x) findzstar(x,z,f_d,dz,rho_hh,delta,gamma,rho,alpha,epsilon,1);
        zstar        = fsolve(findzstar2,zstar,opt);
        Aux          = findzstar(zstar,z,f_d,dz,rho_hh,delta,gamma,rho,alpha,epsilon,2);
        Omega        = Aux(1);
        X            = Aux(2);

        % 3- Define aux variable X, xstar and Z
        Xaux      = X;
        xstar     = (1-Omega)*gamma;
        Zaux      = (gamma*Xaux)^alpha;

        % 4- Get stock A, K and D
        A       = (((rho_hh+delta))/(alpha*mbar*Zaux*L^(1-alpha)*zstar/(gamma*X)))^(1/(alpha-1));
        K       = xstar*A;
        D       = K-A;

        % 5- Get wages, interest rates and the aux variable varphi
        w      = (1-alpha)*mbar*Zaux*A^alpha*L^(-alpha);
        varphi = alpha*((1-alpha)/w)^((1-alpha)/alpha)*mbar^(1/alpha);

        % 5.5- Get consumption
        Y      = Zaux*A^alpha*L^(1-alpha);
        T      = ((1-mbar)*Y) - thetaP/2*pi^2*Y + share_d*A;
        C      = w*L+(R-delta)*D+T;

        % 6- Get savings
        s(:)    = (gamma*max(z(:)*varphi-R,0)+R-delta-rho);
        shat(:) = s(:); %  shat=(s-Adot/A), in SS Adot=0

        % 7- Construct B matrix
        beta_add(:) = shat(:);


        % B matrix:
        Bb = spdiags(beta_add(:),0,J,J)+B_base;

        % To check it is well computed: sum(B,1) needs to be = to shat(:)
        Check=full(Bb); sum_check=sum(Check,1); B_error=max(abs(sum_check(:)-shat(:)));
        if (B_error>1e-10)
            disp('-- B Matrix not well computed ---')
            disp(B_error)
            break
        end

        B          = Bb;
        Baux       = (speye(J)-Deltat*B);
        ff    = spdiags(1+Deltat*shat(:),0,J,J)*f_d;  %savings explicit
        ff    = (speye(J)-Deltat*B_base)\ff;        %brownian motion implicit
        f_sum = sum(ff)*dz;
        fnew  = ff./f_sum;
        fnew  = ff./1;
       
        % Convergenge criteria
        fchange = (fnew - f_d);
        dist(n) = max(abs(fchange));
        dist(n);
        n;

        if dist(n)<tol 
            if pi==0
                disp('Wealth shares Converged, Iteration = ')
                disp(n)
            end
            f_d=fnew;
            break
        end

        if n==maxit
            disp('-- Wealth shares did NOT converge --')
            pi
            dist(n)
        end


    end

end
%----------------------------------------------------------------

%----------------------------------------------------------------
function out=residual(x_in) % to find L; calls the SS_give_L function
    L=x_in;
    SS_given_L;
    out=L- (Upsilon/w*C)^(1/-psi);
end
%----------------------------------------------------------------

%----------------------------------------------------------------
function [ resid ] = findzstar(zstar,z,f_d,dz,rho_hh,delta,gamma,rho,alpha,epsilon,ind)
    
    loc_star  = find(z<zstar,1,'last');
    
    if (zstar<z(1));loc_star=1;end %extrapolate
    if (zstar>z(end));loc_star=size(f_d,1)-1;end %extrapolate
    
    Omega=sum(f_d(z<zstar)*dz)...
                +((f_d(loc_star)+0.5*(zstar-z(loc_star))/dz*(f_d(loc_star+1)-f_d(loc_star)))*(zstar-z(loc_star))/dz-f_d(loc_star)/2)*dz;
            
    X=sum(f_d(z>=zstar).*dz.*z(z>=zstar))...
                +((f_d(loc_star+1)*z(loc_star+1)+0.5*(z(loc_star+1)-zstar)/dz*(f_d(loc_star)*z(loc_star)-f_d(loc_star+1)*z(loc_star+1)))*(z(loc_star+1)-zstar)/dz-f_d(loc_star+1)*z(loc_star+1)/2)*dz; 
    
    if ind==1
        resid=((rho_hh+delta)*gamma*(1-Omega)+(rho-rho_hh)).*zstar./(gamma*X)-(rho_hh+delta);
    else
        resid=[Omega,X];        
    end
end
%----------------------------------------------------------------


end


