%------------------------------------------------------------------------
%
% model_parameters.m
%
% October 2021
%
% Replication codes: "Monetary Policy and Capital Misallocation"
%
% (c) Beatriz Gonzalez, Galo Nuno, Dominik Thaler and Silvia Albrizio
% 
% This file sets up the parameters of the model
%
%------------------------------------------------------------------------

global params

% Choose here if we are doing timeless exercise or time 0 ------
timeless = 1;      % timeless (1) or time 0 (0)


% -- PARAMETERS --------------------
varpsi   = 0.1;       %  Fraction of firms' assets at entry
eta      = 0.12;      %  Firms' death rate
rho      = 0.112;     %  eta*(1-varpsi) - only need this later in the code
rho_hh   = 0.025;     %  Rate of time preference of HH
delta    = 0.065;     %  Capital depreciation rate
gamma    = 1.4;       %  Borrowing constraint parameter
alpha    = 0.3;       %  Capital share in production
L        = 1;         %  Normalization of labor supply Ls=1
Upsilon  = 1;         %  Labor weight - endogenized st L=1  - this value is just an initialization
thetaP   = 100;       %  Price adjustment cost
epsilon  = 10;        %  Markup
phiTR    = 1.25;      %  Taylor rule coefficient (for general equilibrium exercises) 
rhoTR    = 0.8 ;      %  Inertia in TR: quarterly DT = 0.8, CT (1-0.8)/(1/4)=0.8
pibar    = 0;         %  SS inflation
eta_q    = .2;        %  TFP shock persistence, yearly DT = 0.8, CT (1-0.8)/1=0.2
Deltat   = 1/4;       %  Time period
alphaZ   = alpha;     %  exponent on exogenous and endogenous productivity in production function.
share_d  = rho;       %  Dividends to HH after exit.
psi      = 1;         %  Inverse Frisch elasticity
phiK     = 10;        %  Capital adj costs

IP = 0; % Indicator: monopolistic competition profits go to HH (IP=0) or to firms (IP=1)

N_period  = 600/Deltat; % Number of periods to have 200 years

% -- GRIDS -------------------------
J    = 200;                    % number of grid points of productivity z
zmin = 0.0001;                 %  min grid z
zmax = 5.2;                    %  max grid z
z    = linspace(zmin,zmax,J)'; % Equally spaced grids
dz   = (zmax-zmin)/(J-1);      % Distance between grids
dz2  = dz^2;

%%% -- OU PROCESS   --------------------
%%% log z from the data (discrete time) is an AR(1)
%%% log(z_t+1)=corr*log(z_t)+u, u ~ N(0,sig_d) ; sig_d2= sig_d^2
corr      = 0.8;               %  Quarterly persistence of the discrete shock process (quarterly = 0.8; Gilchrist, Zakrajsek, Sim 2014)
corr_dt   = 1/4;               %  Frequency of corr --> quarterly
sig_d     = 0.3;               %  annual std deviation shock of the discrete shock process in logs (Gilchrist, Zakrajsek, Sim 2014)

% Convert process into continuous time 
% dlog(z) = -theta*log(z)dt + sig2*dW --> assumes logzmean=0
theta   = (1-corr)/corr_dt;    %  drift in logs is theta_z
sig     = sig_d;               %  the std dev of the shock of the process logs
sig2    = sig^2;               %  variance of the shock of the process logs

% % % Using Ito's Lemma, transform the OU process of log z into levels
% % % dz = (-theta_z*log(z) + sig2/2) z dt + z^2 sig2 dW
mu    = (-theta*log(z) + sig2/2).*z;      % drift of process in levels 
s2    = z.^2.*sig2;                       % variance of process in levels
s1    = s2.^0.5;                          % std dev of process in levels

% Preliminaries for B matrix --> Constant for all iterations
varrho=zeros(J,1);xi=zeros(J,1);beta_base=zeros(J,1);
varrho(:)    = mu(:)/dz+s2(:)/(2*dz2);
xi(:)        = s2(:)/(2*dz2);
beta_base(:) = -mu(:)/dz-s2(:)/(dz2);


% find Upsilon so that L=1 for pi=0
params     =[[rho rho_hh delta gamma alpha psi sig theta Deltat dz Upsilon thetaP epsilon phiTR pibar eta_q timeless share_d rhoTR phiK J N_period IP]'; z; varrho; xi; beta_base];









